// GLOBAL VARIABLES
var map, wpsObj, urlWPS, obsPoint;

// URLs change according to needs
var urlWPS="http://geo102.fsv.cvut.cz:8080/fgis/services/wps";

// OPTION GRITTER
$.extend($.gritter.options,{position:'bottom-right',fade_in_speed:'medium',fade_out_speed:2000,time:2000});

/*
 * CREATION OF WPS OBJECT, CALL OF WPS OBJECT
 * 
 */

function callWPS(){
    // Creation of WPS object
    // EVENTS: onSucceed and onFailed
    // EVENT LIST: "ProcessAccepted", "ProcessSucceeded", "ProcessFailed", "ProcessStarted", "ProcessPaused"
    wpsObj = new OpenLayers.WPS(urlWPS, {onSucceeded: onExecuted, onFailed: onError});
    
    // Setting inputs
    var obsXInput = new OpenLayers.WPS.LiteralPut({
        identifier : "obs_x",
        value: obsPoint.x 
    });
    var obsYInput = new OpenLayers.WPS.LiteralPut({
        identifier : "obs_y",
        value: obsPoint.y
    });

    // Setting outputs
    
    // Note: No format object in output will make an exception in the
    // WPS-GRASS-Bridge
    // The error return is not clear but the problem is the lack of
    // information on how the output should be

    if ($('input[name=crashON]').is(':checked')){
        // crash
        var outputPNG = new OpenLayers.WPS.ComplexPut({identifier:"output", asReference:true});
    } else {
        // ok request
        var outputPNG = new OpenLayers.WPS.ComplexPut({identifier:"output", asReference:true,
                                                       format: {mimeType:'image/png'}});
    }
    
    // Creating Process
    var processName = "viewshed"
    viewshedProcess = new OpenLayers.WPS.Process({identifier: processName,
                                                  inputs: [obsXInput, obsYInput],
                                                  async: false,
                                                  outputs: [outputPNG]});

    // Adding process to WPS object - one WPS object may have multiple
    // processes
    wpsObj.addProcess(viewshedProcess);
    wpsObj.execute(processName);
}

/*
 * WPS events
 */

// Everything went ok 
function onExecuted(process) {
    resultURL = process.getOutput("output").value;

    sizex = sizey = 10000 / 25
    var viewshedLayer = new OpenLayers.Layer.Image("viewshed", resultURL,
                                                   new OpenLayers.Bounds(obsPoint.x - 5000, obsPoint.y - 5000,
                                                                         obsPoint.x + 5000, obsPoint.y + 5000),
                                                   new OpenLayers.Size(sizex, sizey),
                                                   { resolutions: map.baseLayer.resolutions, visibility: true, isBaseLayer: false });
    map.addLayer(viewshedLayer);
}

// WPS exception
function onError(process){
    textData="Error Code:" + process.exception.code + "<br />" + "Text:" + process.exception.text;
    triggerGritter(textData);
};

/*
 * GENERIC FUNCTIONS
 */
function getQueryString(){
    //something like: echo "1 -1027322.43 4684021.24 235409.49 5074577.74" of v.net.path
    //id start_point_x start_point_y end_point_x end_point_y
    queryString="1 ";
    queryString+=startPoint.x+" "+startPoint.y+" ";
    queryString+=endPoint.x+" "+endPoint.y;
    return queryString;
}

function getClosestPoint(featureObj,pixelObj){
    //nodesLayer
    var distanceArr=new Array();
    $.each(nodesLayer.features,function(index,iterFeature){
        
	distanceArr.push([featureObj.geometry.distanceTo(iterFeature.geometry),iterFeature.id]);
    });
    var closestId=distanceArr[0]; //closestId-> [distance,id]
    var len=distanceArr.length;
    for (var i=1;i<len;i++){ 
	if (distanceArr[i][0]< closestId[0]) {closestId=distanceArr[i];}; 
    }
    closestPoint=nodesLayer.getFeatureById(closestId[1]);
    return closestPoint;
}

function triggerGritter(text){
    $.gritter.add({title:"Status:",text:"<p align='center' class='textGitter'>"+text+"<\p>"});
};

/*
 *  ON DOM LOAD QUICKSTART EVERYTHING
 */

$(document).ready(function() {
    var options = { projection: new OpenLayers.Projection('EPSG:3857'), // 3857
                    units: 'm',
                    //restrictedExtent : new OpenLayers.Bounds(1346447.64,6641839.26,2071107.17,6156958.56)
                  };

    //var proj = new OpenLayers.Projection("EPSG:3035");

    map = new OpenLayers.Map('map1', options);

    // base layer
    // gphyLayer = new OpenLayers.Layer.Google("Google Physical",{type: google.maps.MapTypeId.TERRAIN,isBaseLayer:true,sphericalMercator:true});
    gphyLayer = new OpenLayers.Layer.OSM("OSM");

    // observation point layer
    obsLayer = new OpenLayers.Layer.Vector("Observation Point");
    // 50.5548211N, 13.9312608E
    // 1550820.86|6542921
    obsPoint = new OpenLayers.Geometry.Point(1550820.86,6542921.33);
    
    // style observation point map of obsLayer
    var vectorStyle= new OpenLayers.Style({externalGraphic:'${icon}', pointRadius: 15});
    obsLayer.styleMap = new OpenLayers.StyleMap({'default':vectorStyle}); 
    obsLayer.addFeatures([new OpenLayers.Feature.Vector(obsPoint, {icon: "./css/images/flag-finish.png"})]);
    
    // add everything to map
    map.addLayers([gphyLayer, obsLayer]);
    map.setCenter(new OpenLayers.LonLat(1550820.86, 6542921.33), 14);
    
    // drag feature control here is where wps is called !!!!
    controlDrag = new OpenLayers.Control.DragFeature(obsLayer,
                                                     {onComplete: function(featureObj, pixelObj) {
                                                         //	                                                 closestFeature = getClosestPoint(featureObj,pixelObj)
	                                                 //featureObj.geometry.x = closestFeature.geometry.x;
	                                                 //featureObj.geometry.y = closestFeature.geometry.y;
	                                                 obsLayer.redraw();
	                                                 // remove old layer if necessary
	                                                 $.each(map.getLayersByName("viewshed"), function(index, iterLayer) { map.removeLayer(iterLayer); } );
	                                                 // calling WPS
	                                                 triggerGritter("Calling WPS");
	                                                 callWPS();
	                                             } // end function
	                                             } // end onComplete
	                                            ); //end controlDrag
    
    // add control to map 
    map.addControl(controlDrag);
    controlDrag.activate();
        
    triggerGritter("Move flag to trigger WPS")
    
    // default option, no crash !!!
    $('input[name=crashON]').attr('checked', false);
}); //end of document ready
